package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class XTEAEngine implements BlockCipher {
   private static final int rounds = 32;
   private static final int block_size = 8;
   private static final int key_size = 16;
   private static final int delta = -1640531527;
   private static final int d_sum = -957401312;
   private int[] _S = new int[4];
   private boolean _initialised = false;
   private boolean _forEncryption;

   private void setKey(byte[] key) {
      this._S[0] = this.bytesToInt(key, 0);
      this._S[1] = this.bytesToInt(key, 4);
      this._S[2] = this.bytesToInt(key, 8);
      this._S[3] = this.bytesToInt(key, 12);
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = this.bytesToInt(in, inOff);
      int in1 = this.bytesToInt(in, inOff + 4);
      inOff = 0;

      for(int var6 = 0; var6 != 32; ++var6) {
         int[] in2;
         int var10001 = var5 += (in1 << 4 ^ in1 >>> 5) + in1 ^ inOff + (in2 = this._S)[inOff & 3];
         inOff += -1640531527;
         in1 += (var10001 << 4 ^ var5 >>> 5) + var5 ^ inOff + in2[inOff >>> 11 & 3];
      }

      this.unpackInt(var5, out, outOff);
      int var7 = outOff + 4;
      this.unpackInt(in1, out, var7);
      return 8;
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = this.bytesToInt(in, inOff);
      int in1 = this.bytesToInt(in, inOff + 4);
      inOff = -957401312;

      int var10000;
      for(int var6 = 0; var6 != 32; in1 = var10000) {
         var10000 = var5;
         int[] in2;
         int var10001 = var5 = in1 - ((var5 << 4 ^ var5 >>> 5) + var5 ^ inOff + (in2 = this._S)[inOff >>> 11 & 3]);
         inOff -= -1640531527;
         in1 = var10000 - ((var10001 << 4 ^ var5 >>> 5) + var5 ^ inOff + in2[inOff & 3]);
         ++var6;
         var10000 = var5;
         var5 = in1;
      }

      this.unpackInt(var5, out, outOff);
      int var7 = outOff + 4;
      this.unpackInt(in1, out, var7);
      return 8;
   }

   private int bytesToInt(byte[] in, int inOff) {
      int var3 = inOff + 1;
      return in[inOff] << 24 | (in[var3++] & 255) << 16 | (in[var3++] & 255) << 8 | in[var3] & 255;
   }

   private void unpackInt(int v, byte[] out, int outOff) {
      int var10001 = v;
      int var10003 = v;
      int var10005 = v;
      int var4 = outOff + 1;
      out[outOff] = (byte)(v >>> 24);
      v = var4 + 1;
      out[var4] = (byte)(var10005 >>> 16);
      var4 = v + 1;
      out[v] = (byte)(var10003 >>> 8);
      out[var4] = (byte)var10001;
   }

   public String getAlgorithmName() {
      return "XTEA";
   }

   public int getBlockSize() {
      return 8;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this._forEncryption = forEncryption;
         this._initialised = true;
         this.setKey(((KeyParameter)params).getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to TEA init - " + params.getClass().getName());
      }
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this._initialised) {
         if (inOff + 8 <= in.length) {
            if (outOff + 8 <= out.length) {
               return this._forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
      }
   }

   public void reset() {
   }
}
